#!/usr/bin/ruby
# encoding: utf-8
#
# (c) 2011 Mattias Schlenker for Heise Verlag, Hannover, Germany
# This script is part of desinfec't 2011, update 1


# Analyze the log from bdscan
#
# Use a Hash to store information about found viruses
# [ filename, archivepath ] => [ avira, bitdefender, clamav, kaspersky ]

@scanners = [
	[ "avira", "Avira", true, "/AntiVir/scancl", 
		"/opt/desinfect/update_avira.sh", "/opt/desinfect/run_meta_avira.sh" ],
#	[ "bitdefender", "Bitdefender", true, "/usr/bin/bdscan", 
#		"/opt/desinfect/update_bitdefender.sh", "/opt/desinfect/run_meta_bitdefender.sh" ],
	[ "eset", "ESET", true, "/opt/eset/esets/sbin/esets_scan",
		"/opt/desinfect/update_eset.sh", "/opt/desinfect/run_meta_eset.sh" ],
	[ "f-secure", "F-Secure", true, "/opt/f-secure/fssp/bin/fsav",
		"/opt/desinfect/update_f-secure.sh", "/opt/desinfect/run_meta_f-secure.sh" ],
	[ "kaspersky", "Kaspersky", false, "/usr/lib/kl/kav",
                "/opt/desinfect/update_kaspersky.sh", "/opt/desinfect/run_meta_kaspersky.sh" ],
	[ "sophos", "Sophos", false, "/opt/sophos-av/bin/savscan",
                "/opt/desinfect/update_sophos.sh", "/opt/desinfect/run_meta_sophos.sh" ],
	[ "clamav", "ClamAV", false, "/usr/bin/clamscan", 
		"/opt/desinfect/update_clamav.sh", "/opt/desinfect/run_meta_clamav.sh" ]
]

@cfgfile = "metascan.cfg"
@cfgdir = "/opt/desinfect/signatures/config"
@scan_archives = false
@automount = true
@active_scanners = Array.new
@scanners.each { |s| @active_scanners.push(s[0]) if s[2] == true } 
@startdir = "/media"

# config file: /opt/desinfect/signatures/config/metascan.cfg
# 
# archives=false
# scanners=clamav bitdefender
# automount=true
# startdir=/media

def read_config
	if File.exists?("#{@cfgdir}/#{@cfgfile}") 
		File.open("#{@cfgdir}/#{@cfgfile}").each { |line|
			ltoks = line.split("=") 
			puts "read cfg: " + line.strip
			if ltoks[0] == "archives"
				@scan_archives = true if ltoks[1].strip.downcase == "true" 
			elsif ltoks[0] == "scanners" 
				@active_scanners = ltoks[1].strip.split
				puts @active_scanners.join(", ")
			elsif ltoks[0] == "automount" 
				@automount = false if ltoks[1].strip.downcase == "false" 
			elsif ltoks[0] == "startdir"
				@startdir = ltoks[1].strip
			end
		}
	end
end

# Force read of configuration...
read_config

found_virii = Hash.new
archives = Array.new
success = [ false, false, false, false ]

lang = "de"
lang = "nl" if ENV["LANG"] =~ /^nl/

# encoding: utf-8
csv_header = Hash.new
csv_header["de"] = "\"Infizierte Datei\";\"ggf. Datei in Archiv\";\"Fund durch Avira\";\"Fund durch ESET\";\"Fund durch ClamAV\";\"Fund durch Kaspersky\";\n\"\";\"\";\"\";\"\";\"\";\"\";\n" 
csv_head_start = Hash.new
csv_head_start["de"] = "\"Infizierte Datei\";\"ggf. Datei in Archiv\";"
csv_single_head = Hash.new
csv_single_head["de"] = "\"Fund durch SCANNER\";"
csv_header["nl"] = "\"Geïnfecteerd bestand\";\"evt. bestand in archief\";\"Gevonden door Avira\";\"Gevonden door ESET\";\"Gevonden door ClamAV\";\"Gevonden door Kaspersky\";\n\"\";\"\";\"\";\"\";\"\";\"\";\n" 
csv_head_start["nl"] = "\"Geïnfecteerd bestand\";\"evt. bestand in archief\";"
csv_single_head["nl"] = "\"Gevonden door SCANNER\";"
html_title = Hash.new
html_title["de"] = "Virenfunde"
html_title["nl"] = "Rapport"
html_header = Hash.new
html_header["de"] = "<tr>\n<th class=\"leftempty\">&nbsp;</th>\n<th>Avira</th>\n<th>ESET</th>\n<th>ClamAV</th>\n<th>Kaspersky</th>\n<th>Aktion</th>\n</tr>\n"
html_header["nl"] = "<tr>\n<th class=\"leftempty\">&nbsp;</th>\n<th>Avira</th>\n<th>ESET</th>\n<th>ClamAV</th>\n<th>Kaspersky</th>\n<th>Actie</th>\n</tr>\n"
action_fm = Hash.new
action_fm["de"] =  "Dateimanager"
action_fm["nl"] =  "Bestandsbeheer"
action_rename = Hash.new
action_rename["de"] =  "umbenennen"
action_rename["nl"] =  "hernoemen"
action_all = Hash.new
action_all["de"] = "Alle gefundenen Dateien mit der Endung .VIRUS versehen"
action_all["nl"] = "Alle gevonden bestanden laten eindigen op .VIRUS"
action = Hash.new
action["de"] = "Aktion"
action["nl"] = "Actie"
msg_nothing = Hash.new
msg_nothing["de"] = "Es wurde keine Schadsoftware auf den untersuchten Laufwerken gefunden."
msg_nothing["nl"] = "We konden geen schadelijke software vinden op de onderzochte drives."
msg_interrupt = Hash.new
msg_interrupt["de"] = "Die Logdateien von einem oder mehreren Virenscannern sind unvollständig. Dies deutet auf einen Abbruch aus Speichermangel oder durch Nutzereingriff hin. Starten Sie den Virenscan gegebenenfalls erneut ohne die Untersuchung von Archiven."
msg_interrupt["nl"] = "De logbestanden van een of meerdere virusscanners zijn niet compleet. Dit duidt erop dat het proces is afgebroken door gebrek aan geheugen of door ingrijpen van de gebruiker. Start de virusscanner eventueel opnieuw, maar dan zonder controle van archieven."
avira_link = Hash.new
avira_link["de"] = "http://www.avira.com/de/support-virus-lab?sq="
avira_link["nl"] = "http://www.avira.com/en/support-virus-lab?sq="
congrats = Hash.new
congrats["de"] = "Gratuliere!\n\nEs wurde keine Schadsoftware auf den untersuchten Laufwerken gefunden!"
congrats["nl"] = "Gefeliciteerd!\n\nEr is geen schadelijke software op de onderzochte schijven gevonden!"
fmlong = Hash.new
fmlong["de"] = "Im Dateimananger öffnen"
fmlong["nl"] = "In Bestandsbeheer openen" 
scandone = Hash.new

#if File.exist?("/tmp/desinfect_logs/LOG_bitdefender.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /bitdefender/i )
#File.open("/tmp/desinfect_logs/LOG_bitdefender.log").each { |line|
#	# puts line
#	if line =~ /^\//
#		ltoks = line.split("\t")
#		if ltoks[1] =~ /^infected/
#			puts line 
#			filename = ltoks[0]
#			archivepath = nil
#			# puts "infected!"
#			# 
#			if ltoks[0] =~ /=>/
#				# puts "archive found"
#				ptoks = ltoks[0].split("=>")
#				filename = ptoks[0]
#				archivepath = ptoks[1]
#			end
#			found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil ] unless 
#				found_virii.has_key?( [filename, archivepath] )
#			found_virii[ [filename, archivepath] ][1] = ltoks[1].gsub("infected:", "").strip
#		end
#	elsif line =~ /^Results:/
#		success[1] = true
#		$stderr.puts "bdscan successfully terminated"
#	end
#}
#else
#	success[1] = true
#end

logdir = "/tmp/desinfect_logs"
logdir = "/opt/desinfect/signatures/desinfect_logs" if system("mountpoint -q /opt/desinfect/signatures")  
logdir = "/tmp/desinfect_logs" unless File.directory?(logdir) 

if File.exist?("#{logdir}/LOG_kaspersky.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /kaspersky/i )
File.open("#{logdir}/LOG_kaspersky.log").each { |line|
	if line =~ /^;/
		# puts "Comment found, ignoring"
		success[3] = true if line =~ /--- Statistics ---/
	else
		# puts line
		ltoks = line.split("\t")
		if ltoks[1].strip =~ /^\// && ( ltoks[2].strip == "detected" || ltoks[2].strip =~ /^suspic/ )
			puts line
			virus = ltoks[3].strip
			filename = ltoks[1].strip
			archivepath = nil
			if ltoks[1] =~ /\/\//
				atoks = ltoks[1].split("//")
				puts "ARCHIVE: " + atoks[0]
				puts "FILE:    " + atoks[1] 
				filename = atoks[0]
				archivepath = atoks[1].strip
			end
			found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
				found_virii.has_key?( [filename, archivepath] )
			found_virii[ [filename, archivepath] ][3] = virus
		end
	end
}
else
	success[3] = true
end

if File.exist?("#{logdir}/LOG_clamav.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /clamav/i )
File.open("#{logdir}/LOG_clamav.log").each { |line|
	# puts line
	if line =~ /^\//
		ltoks = line.strip.split(": ")
		filename = ltoks[0]
		archivepath = nil
		if ltoks[1] =~ /FOUND$/
			# puts ltoks[1]
			puts line
			found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
				found_virii.has_key?( [filename, archivepath] )
			found_virii[ [filename, archivepath] ][2] = ltoks[1].sub(/FOUND$/, "").strip
		end
	elsif line =~ /-\sSCAN\sSUMMARY\s-/
		success[2] = true
		$stderr.puts "clamscan successfully terminated"
	end	
}
else
	success[2] = true
end

if File.exist?("#{logdir}/LOG_avira.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /avira/i )
File.open("#{logdir}/LOG_avira.log").each { |line|
	if line =~ /ALERT\:\s\[(.*?)\]\s(.*?)\s\<<</
		begin
			puts line
		rescue
		end
		virus = $1
		filename = $2
		archivepath = nil
		if filename =~ /-->/
			ptoks = filename.split(" --> ")
			filename = ptoks[0]
			archivepath = ptoks[1]
		end
		found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
			found_virii.has_key?( [filename, archivepath] )
		found_virii[ [filename, archivepath] ][0] = virus
	elsif line =~ /^Statistics\s:/
		success[0] = true
		$stderr.puts "avira successfully terminated"
	end
}
else
	success[0] = true
end

if File.exist?("#{logdir}/LOG_eset.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /eset/i )
	File.open("#{logdir}/LOG_eset.log").each { |line|
	if line =~ /name=\"(.*?)\",\sthreat=\"(.*?)\",/
		begin
			puts line
		rescue
		end
		virus = $2
		filename = $1
		unless virus.strip == ""
			archivepath = nil
			if filename =~ / » /
				ptoks = filename.split(" » ")
				filename = ptoks[0]
				archivepath = ptoks[-1] 
			end
			found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
				found_virii.has_key?( [filename, archivepath] )
			found_virii[ [filename, archivepath] ][1] = virus
		end
	elsif line =~ /^Scan completed at:/
		success[1] = true
		$stderr.puts "eset successfully terminated"
	end
	}
else
	success[1] = true
end

if File.exist?("#{logdir}/LOG_f-secure.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /f-secure/i )
	File.open("#{logdir}/LOG_f-secure.log").each { |line|
	if line =~ /^\[(.*?)\] (.*?): Infected: (.*?) \[/
		virus = $3
		filename = $1
		archivepath = $2
		if line =~ /^\[(.*?)\] \[(.*)\] (.*?): Infected: (.*?) \[/
			archivepath = $3
		end
		found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
			found_virii.has_key?( [filename, archivepath] )
		found_virii[ [filename, archivepath] ][4] = virus
	elsif line =~ /^(.*?): Infected: (.*?) \[/
		virus = $2
		filename = $1
		archivepath = nil
		found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
			found_virii.has_key?( [filename, archivepath] )
		found_virii[ [filename, archivepath] ][4] = virus
	elsif line =~ /^Scan ended at/
		success[4] = true
		$stderr.puts "f-secure successfully terminated"
	end
	}
else
	success[4] = true
end

if File.exist?("#{logdir}/LOG_sophos.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /sophos/i )
	File.open("#{logdir}/LOG_sophos.log").each { |line|
	if line =~ />>> Virus \'(.*?)\' found in file (.*?)$/
		virus = $1
		filename = $2
		archivepath = nil
		unless File.exists? filename
			line =~ />>> Virus \'(.*?)\' found in file (.*)\/(.*?)$/
			if File.exists? $2
				filename = $2
				archivepath = $3 
			end
		end
		found_virii[ [filename, archivepath] ] = [ nil, nil, nil, nil, nil, nil ] unless 
			found_virii.has_key?( [filename, archivepath] )
		found_virii[ [filename, archivepath] ][5] = virus
	elsif line =~ /^End of S/
		success[5] = true
		$stderr.puts "f-secure successfully terminated"
	end
	}
else
	success[5] = true
end

outfile = File.open("#{logdir}/metascan.csv", "w")
htmfile = File.open("#{logdir}/metascan.html", "w")
system("cp /opt/desinfect/metascan.css #{logdir}")
htmfile.write("<html>\n<head>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"> <title>" + html_title[lang] + "</title>\n")
htmfile.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"./metascan.css\" />\n</head>\n")
htmfile.write("<body>\n<h2 align=\"center\">" + html_title[lang] + "</h2>\n")
htmfile.write("<table class=\"virustab\">\n")
# outfile.write("\"Infizierte Datei\";\"ggf. Datei in Archiv\";\"Fund durch Avira\";\"Fund durch BitDefender\";\"Fund durch ClamAV\";\"Fund durch Kaspersky\";\n\"\";\"\";\"\";\"\";\"\";\"\";\n")
outfile.write csv_head_start[lang]
# htmfile.write("<tr>\n<th class=\"leftempty\">&nbsp;</th>\n<th>Avira</th>\n<th>BitDefender</th>\n<th>Kaspersky</th>\n<th>ClamAV</th>\n<th>Aktion</th>\n</tr>\n") 
htmfile.write("<tr>\n<th class=\"leftempty\">&nbsp;</th>\n")

if File.exist?("#{logdir}/LOG_avira.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /avira/i )
	outfile.write(csv_single_head[lang].gsub("SCANNER", "Avira"))
	htmfile.write("<th>Avira</th>\n")
else
	outfile.write("\"\";")
	htmfile.write("<th></th>\n")
end
if File.exist?("#{logdir}/LOG_eset.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /eset/i )
	outfile.write(csv_single_head[lang].gsub("SCANNER", "ESET"))
	htmfile.write("<th>ESET NOD32</th>\n")
else
	outfile.write("\"\";")
	htmfile.write("<th></th>\n")
end
if File.exist?("#{logdir}/LOG_f-secure.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /f-secure/i )
	outfile.write(csv_single_head[lang].gsub("SCANNER", "F-Secure"))
	htmfile.write("<th>F-Secure</th>\n")
else
	outfile.write("\"\";")
	htmfile.write("<th></th>\n")
end
if File.exist?("#{logdir}/LOG_kaspersky.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /kaspersky/i )
	outfile.write(csv_single_head[lang].gsub("SCANNER", "Kaspersky"))
	htmfile.write("<th>Kaspersky</th>\n")
else
	outfile.write("\"\";")
	htmfile.write("<th></th>\n")
end
if File.exist?("#{logdir}/LOG_sophos.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /sophos/i )
	outfile.write(csv_single_head[lang].gsub("SCANNER", "Sophos"))
	htmfile.write("<th>Sophos</th>\n")
else
	outfile.write("\"\";")
	htmfile.write("<th></th>\n")
end
if File.exist?("#{logdir}/LOG_clamav.log") && ( ARGV[0].to_s.size < 1 || ARGV[0] =~ /clamav/i )
        outfile.write(csv_single_head[lang].gsub("SCANNER", "ClamAV"))
        htmfile.write("<th>ClamAV</th>\n")
else
        outfile.write("\"\";")
        htmfile.write("<th></th>\n")
end

outfile.write("\n\"\";\"\";\"\";\"\";\"\";\"\";\n")
htmfile.write("<th>" + action[lang] + "</th>\n</tr>\n")

oddeven = 0
found_virii.each { |k, v|
	$stderr.puts k[0] + " => " + k[1].to_s + " eset: " + v[1].to_s + " kav: " + v[3].to_s + " clam: " + v[2].to_s + " avira: " + v[0].to_s
	outfile.write("\"" + k[0] + "\";\"" + k[1].to_s + "\";\"" + v[0].to_s + "\";\"" + v[1].to_s + "\";\"" + v[2].to_s + "\";\"" + v[3].to_s + "\";\n")
	if oddeven % 2 == 0
		htmfile.write("<tr class=\"even\">\n<td colspan=\"8\" class=\"filename\"><a title=\"#{fmlong[lang]}\" href=\"filemanager://" + k[0]   + "\">" + k[0]  + "</a>" )
	else
		# htmfile.write("<tr class=\"odd\">\n<td colspan=\"6\" class=\"filename\">" + k[0])
		htmfile.write("<tr class=\"odd\">\n<td colspan=\"8\" class=\"filename\"><a title=\"#{fmlong[lang]}\" href=\"filemanager://" + k[0]   + "\">" + k[0]  + "</a>" )
	end
	# htmfile.write("<tr>\n<td colspan=\"5\">" + k[0])
	htmfile.write(" (" + k[1].to_s + ")") if k[1].to_s.strip.size > 0
	if oddeven % 2 == 0
		htmfile.write("</td>\n</tr>\n<tr  class=\"even\">\n<td class=\"leftempty\"></td>\n<td>")
	else
		htmfile.write("</td>\n</tr>\n<tr  class=\"odd\">\n<td class=\"leftempty\"></td>\n<td>") 
	end
	htmfile.write("<a target=\"_blank\" href=\"" + avira_link[lang] + v[0].to_s + "\">" + v[0].to_s) if v[0].to_s.size > 0
	htmfile.write("</td>\n<td>")
	htmfile.write("<a target=\"_blank\" href=\"http://www.virusradar.com/en/search/all/" + v[1].to_s + "\">" + v[1].to_s) if v[1].to_s.size > 0
	htmfile.write("</td>\n<td>")
	htmfile.write("<a target=\"_blank\" href=\"http://cgi.f-secure.com/cgi-bin/websearch2/vsearch.cgi?q=" + v[4].to_s + "\">" + v[4].to_s) if v[4].to_s.size > 0
	htmfile.write("</td><td>\n")
	htmfile.write("<a target=\"_blank\" href=\"http://www.securelist.com/en/find?chapter=10&behavior=&Search=Search&search_type=2&words=" +
                v[3].to_s + "\">" + v[3].to_s) if v[3].to_s.size > 0
        htmfile.write("</td><td>\n")
	htmfile.write("<a target=\"_blank\" href=\"https://www.sophos.com/en-us/search-results.aspx?search=" + v[5].to_s + "\">" + v[5].to_s) if v[5].to_s.size > 0
	htmfile.write("</td>\n<td class=\"clamlight\">")
	htmfile.write("<a target=\"_blank\" href=\"http://www.google.com/search?q=" + v[2].to_s + "\">" + v[2].to_s) if v[2].to_s.size > 0
	htmfile.write("</td><td>\n")
	htmfile.write("<a href=\"vtupload://" + k[0] + "\">VirusTotal</a>\n ")
	# htmfile.write("<a href=\"filemanager://" + k[0] + "\">" + action_fm[lang] + "</a>\n ")
	htmfile.write("<a href=\"rename://" + k[0] + "\">" + action_rename[lang] + "</a> ") unless k[0].strip =~ /\.VIRUS$/ 
	# Virustotal
	# if File.exists?(k[0]) && File.size(k[0]) < 67108864
	#	shasum = ` sha256sum "#{k[0]}" `.split[0]  	
	#	htmfile.write("\n<a href=\"https://www.virustotal.com/en/file/#{shasum}/analysis/\" target=\"_blank\">Virustotal</a>\n")
	# end
	htmfile.write("</td>\n</tr>\n")
	oddeven += 1
}
htmfile.write("</table>\n")
if found_virii.size < 1 
	htmfile.write('<div class="nothingfound"><p>' + msg_nothing[lang] + '</p></div>')
else	
	htmfile.write('<div class="cleanall"><p><a href="renameall:///virus">' + action_all[lang] + '</a></p></div>')
end
htmfile.write("</body>\n</html>\n")
outfile.close 
htmfile.close
now = Time.new.strftime("%Y%m%d-%H%M")
unless system("mountpoint -q /opt/desinfect/signatures")
	[ "csv", "html" ].each { |s|
		system("cp -v /tmp/desinfect_logs/metascan.#{s} /tmp/desinfect_logs/metascan-#{now}.#{s}") 
	}
end
if system("mountpoint -q /media/desinfDATA") 
	system("umount /media/desinfDATA")
end
if system("mountpoint -q /media/desinfDATA") 
	system("umount /media/desinfect/desinfDATA")
end
if File.blockdev?("/dev/disk/by-label/desinfDATA")
	system("mkdir -p /media/desinfDATA")
	system("mount -t vfat -o rw,uid=999,gid=999 /dev/disk/by-label/desinfDATA /media/desinfDATA") 
	loopnum = -1
	unless system("mountpoint -q /media/desinfDATA") 
		# Search the correct loopfile
		0.upto(31) { |n|
			lbl = ` blkid -s LABEL -o value /dev/loop#{n} `.strip 
			loopnum = n if lbl =~ /desinfdata/i
		}
	end
	if loopnum > -1
		system("mount -t vfat -o rw,uid=999,gid=999 /dev/loop#{loopnum} /media/desinfDATA") 
	end
	if system("mountpoint -q /media/desinfDATA") 
		system("mkdir /media/desinfDATA/desinfect-logs")
		system("cp /opt/desinfect/metascan.css /media/desinfDATA/desinfect-logs")
		if lang == "nl"
			system("cp -v #{logdir}/metascan.html /media/desinfDATA/desinfect-logs/Scanresultaat-#{now}.html") 
		else
			system("cp -v #{logdir}/metascan.html /media/desinfDATA/desinfect-logs/Scan-Ergebnis-#{now}.html") 
		end
	end
end

if success.include? false
	system("zenity --error --text \"" + msg_interrupt[lang] + "\"") unless ARGV[1].to_s == "silent" 
	exit 1
elsif found_virii.size < 1 
	system("zenity --info --text '#{congrats[lang]}'") 
end
system("touch /var/run/desinfect/analyze_successful") 
exit 0 


