#!/bin/bash
# encoding: utf-8

# (c) 2013 Mattias Schlenker für Heise Zeitschriftenverlag GmbH & Co KG

lang=de
tstyle='+%d.%m.%Y_%H:%M'
termtitle='Bitte nicht schließen!'
failedhead='Signatur-Aktualisierung fehlgeschlagen'
failedtail='Dieser Scanner verwendet veraltete Signaturen.'
somefailed='Die Aktualisierung der Signaturen für die folgenden Scanner ist fehlgeschlagen:'
allsuccess='Alle Signaturen wurden erfolgreich aktualisiert.'
case $LANG in 
	nl*)
		lang=nl
		tstyle='+%d-%m-%Y_%H:%M'
		termtitle='Niet sluiten!'
		failedhead='Bijwerken van signatures mislukt'
		failedtail='Deze scanner gebruikt verouderde signatures.'
		somefailed='De signatures zijn voor de volgende scanner(s) niet bijgewerkt:'
		allsuccess='Alle signatures zijn bijgewerkt.'
	;;
esac

today=` date "+%Y%m%d" ` 

# change for NL 2016 - no kaspersky present 
# scanners="eset avira clamav f-secure"
scanners="eset avira f-secure"
memory=`cat /proc/meminfo | head -n1 | awk '{print $2}' `
if mountpoint -q /opt/desinfect/signatures || [ "$memory" -gt 3000000 ] ; then
	scanners="$scanners sophos-av"
	test -x /usr/lib/kl/kav && scanners="$scanners kaspersky"
fi

if wget -T 3 -O /dev/null http://www.heise.de/ct/desinfect13-check.gif ; then
	echo "Network OK..."
else
	if [ "$lang" = "nl" ] ; then
		zenity --info --title "Signatures bijwerken" --text "Zorg voor een verbinding met internet. Klik vervolgens op \"OK\" om alle signatures bij te werken."
	else
		zenity --info --title "Aktualisierung der Virensignaturen" --text "Bitte stellen Sie sicher, dass Sie über Verbindung zum Internet verfügen. Klicken Sie dann auf \"OK\", um die Aktualisierung aller Signaturen zu starten."
	fi
fi

failed=''
if mountpoint -q /opt/desinfect/signatures ; then
	echo 'Signatures already mounted, skipping...'
else
	sudo /etc/init.d/overlays start
fi
xfce4-terminal --hide-menubar --title="$termtitle" -e "sudo bash /opt/desinfect/update.sh"
for n in $scanners ; do
	sudo rm /var/run/desinfect/skip_${n}_update_${today} 
	xfce4-terminal --hide-menubar --title="$termtitle" -e "sudo bash /opt/desinfect/update_${n}.sh"
	if [ -f /var/run/desinfect/skip_${n}_update_${today} ] ; then
		echo "Success: $n"
	else
		failed="$failed $n"
		zenity --error --text "${failedhead}: $n - ${failedtail}" & 
	fi
done


if [ -z "$failed" ] ; then
	zenity --info --text "${allsuccess}"
else
	zenity --error --text "${somefailed}: ${failed}."
fi
